function [S, Ss] = vdoSalCbCore(Ls, As, mSegs, bins, lamHs, lamPs, FQ, MQ, M, MA, VX, VY, P)
% Computing image salience based on the hypercomplex Fourier transform.
%
% Reference
%   2011 - BMVC - Automatic Salient Object Segmentation Based on Context and Shape Prior
%
% Input
%   F       -  image, h0 x w0 x nC
%   parSal  -  saliency parameter
%     Par   -  segmentation parameter, nL x 3
%     bins  -  #bin, {[8 16 16 4]}, CIE L*, a*, b*, Hue
%     th    -  threshold, {.2}
%     eta   -  eta, {.1}
%     maW   -  #maximum width, {400}
%
% Output
%   S       -  salience, h0 x w0
%
% History
%   create  -  Feng Zhou (zhfe99@gmail.com), 01-03-2009
%   modify  -  Feng Zhou (zhfe99@gmail.com), 07-18-2013

% dimension
nL = length(Ls);
[h, w] = size(FQ);
siz = [h w];

% each level
[S, T1, T2] = zeross(h, w);
Ss = cellss(1, nL);
prCIn('level', nL, 1);
for cL = 1 : nL
    prC(cL);

    FQ = FQ;
    L = Ls{cL};
    MQ = MQ;
    A = As{cL};
    mSeg = mSegs(cL);
    cSegs = 1 : mSeg;

    %% histogram
    HstF = computeRegionHist_fast(FQ, L, prod(bins(1 : 4)), mSeg);
    HstM = computeRegionHist_fast(MQ, L, bins(5), mSeg);
    HstO = computeHof(VX, VY, L, bins(6), mSeg);
    Wi = zeros(h, w) + 10;

    %% spatial prior of each region
    [areas, xDs, yDs] = computeRegionSpa(L, cSegs, mSeg);
    mags = computeRegionMag_fast(L, M, mSeg);
    magAs = computeRegionMag_fast(L, MA, mSeg);
    bks = computeRegionMag_fast(L, P, mSeg);

    %% histogram distance
    DF = computeHistDist(HstF, A);
    DM = computeHistDist(HstM, A);
    DO = computeHistDist(HstO, A);

    %% histogram saliency
    sHstFs = computeSalHst(DF, A, areas, siz);
    sHstMs = computeSalHst(DM, A, areas, siz);
    sHstOs = computeSalHst(DO, A, areas, siz);

    %% spatial prior saliency
    sPriSs = computeSalPriSpa(areas, xDs, yDs, siz)';
    sPriMs = computeSalPriMag(areas, mags);
    sPriAs = computeSalPriMag(areas, magAs);
    sPriBs = computeSalPriMag(areas, bks);

    %% combine
    sHsts = lamHs(1) * sHstFs + lamHs(2) * sHstMs + lamHs(3) * sHstOs;
    sPris = lamPs(1) * sPriSs + lamPs(2) * sPriMs + lamPs(3) * sPriBs + lamPs(4) * sPriAs;
    ss = sHsts' .* sPris;

    %% propagate to pixels
    Si = computePixelSal(ss, L, cSegs);

    if ~any(isnan(Si(:)))
        T1 = T1 + Si .* Wi;
        T2 = T2 + Wi;
    end

    %% store
    Ss{cL} = Si;
end
prCOut(nL);

% propogate the saliency value from region to pixel,
% which is weighted by the color distance, according to Eq.(2)
S = T1 ./ T2;

% normalize
% S = (S - min(S(:))) / (max(S(:)) - min(S(:)) + eps);

% restore
% S = uint8(S * 255);
% S = double(S);

