function [SHstF, SHstM, SHstO, SPriS, SPriM, SPriB, SPriA] = vdoSalVoxA(hrVdo, hrFlow, wsSmp, wsFlow, wsAcc, wsBk, wsSeg, bins, wS, direS)
% Computing video saliency based on spatio-temporal voxels.
%
% Input
%   hrVdo   -  video input handle
%   hrFlow  -  optical flow input handle
%   wsSmp   -  sampling
%   wsFlow  -  flow
%   wsAcc   -  acceleration
%   wsBk    -  background
%   wsSeg   -  segmentation
%   bins    -  bins for quantatize appearance and flow, {[8 16 16 4 8 9]}
%   wS      -  window for cumulating histogram for computing saliency
%   direS   -  window type,  'both' | 'left' | 'right'
%
% Output
%   SHstF   -  salience of apperance, mSeg x nF
%   SHstM   -  salience of magtitude, mSeg x nF
%   SHstO   -  salience of orientation, mSeg x nF
%   SPriS   -  salience of spatial prior, mSeg x nF
%   SPriM   -  salience of motion prior, mSeg x nF
%   SPriB   -  salience of background prior, mSeg x nF
%   SPriA   -  salience of acceleration, mSeg x nF
%
% History
%   create  -  Feng Zhou (zhfe99@gmail.com), 01-03-2009
%   modify  -  Feng Zhou (zhfe99@gmail.com), 07-18-2013

% sampling
[nF0, nF, cFs] = stFld(wsSmp, 'nF0', 'nF', 'pFs');

% flow
magMas = stFld(wsFlow, 'magMas');

% acceleration
magMaAs = stFld(wsAcc, 'magMas');
hrAcc = vdoRIn(wsAcc.vdo, 'comp', 'mat');

% background
hrBk = vdoRIn(wsBk.vdo, 'comp', 'mat');

% segmentation
hrSeg = vdoRIn(wsSeg.vdo, 'comp', 'mat');
[mSeg, A] = stFld(wsSeg, 'mSeg', 'A');

% spatial prior
siz = stFld(hrSeg, 'siz');
[XD, YD] = computeRegionSpaXY(siz);

prIn('vdoSalVoxA', 'bins [%d %d %d %d %d %d], wS %d, direS %s', ...
     bins(1), bins(2), bins(3), bins(4), bins(5), bins(6), wS, direS);

% cache
[Ran0, RanC, idxC] = vdoCacheIdx(cFs, nF0, wS, direS);
[HstFs, HstMs, HstOs, areass, xDss, yDss, Ls, magss, magAss, bkss] = cellss(1, wS);
HstF = [];
iF = 1;

% loop the video
[SHstF, SHstM, SHstO, SPriS, SPriM, SPriB, SPriA] = zeross(mSeg, nF);
prCIn('frame', nF0, .1);
for pF = 1 : nF0
    prC(pF);

    %% useless frame, skip
    iC = idxC(pF);
    if iC < 0
        HstF = [];
        HstFs{-iC} = [];
        continue;
    end

    %% read frame
    F0 = vdoR(hrVdo, pF);
    F = imresize(F0, siz);

    %% read flow
    mat = vdoR(hrFlow, pF);
    [VX, VY] = stFld(mat, 'VX', 'VY');
    VX = double(VX);
    VY = double(VY);

    %% read acc
    mat = vdoR(hrAcc, pF);
    [DX, DY] = stFld(mat, 'DX', 'DY');
    DX = double(DX);
    DY = double(DY);

    %% mag
    M = real(sqrt(VX .^ 2 + VY .^ 2));
    M = M ./ (magMas(pF) + eps);
    MA = real(sqrt(DX .^ 2 + DY .^ 2));
    MA = MA ./ (magMaAs(pF) + eps);

    %% background
    mat = vdoR(hrBk, pF);
    P = double(mat.P);

    %% segmentation
    mat = vdoR(hrSeg, pF);
    L = double(mat.L);
    % S = mat.S;

    %% quantize
    FQ = computeQuantMatrixImg(F, bins(1 : 4));
    MQ = computeQuantMatrixFlow(M, bins(5));

    %% compute histogram
    HstFi = computeRegionHist_fast(FQ, L, prod(bins(1 : 4)), mSeg);
    HstMi = computeRegionHist_fast(MQ, L, bins(5), mSeg);
    HstOi = computeHof(VX, VY, L, bins(6), mSeg);

    %% spatial prior of each region
    [areasi, xDsi, yDsi] = computeRegionSpa_fast(L, XD, YD, mSeg);
    magsi = computeRegionMag_fast(L, M, mSeg);
    magAsi = computeRegionMag_fast(L, MA, mSeg);
    bksi = computeRegionMag_fast(L, P, mSeg);

    %% add to the cumulative
    if isempty(HstF)
        HstF = HstFi;
        HstM = HstMi;
        HstO = HstOi;
        areas = areasi;
        xDs = xDsi;
        yDs = yDsi;
        mags = magsi;
        magAs = magAsi;
        bks = bksi;
    else
        HstF = HstF + HstFi;
        HstM = HstM + HstMi;
        HstO = HstO + HstOi;
        areas = areas + areasi;
        xDs = xDs + xDsi;
        yDs = yDs + yDsi;
        mags = mags + magsi;
        magAs = magAs + magAsi;
        bks = bks + bksi;
    end

    %% remove from the cumulative
    if ~isempty(HstFs{iC})
        HstF = HstF - HstFs{iC};
        HstM = HstM - HstMs{iC};
        HstO = HstO - HstOs{iC};
        areas = areas - areass{iC};
        xDs = xDs - xDss{iC};
        yDs = yDs - yDss{iC};
        mags = mags - magss{iC};
        magAs = magAs - magAss{iC};
        bks = bks - bkss{iC};
    end

    %% update cache
    HstFs{iC} = HstFi;
    HstMs{iC} = HstMi;
    HstOs{iC} = HstOi;
    areass{iC} = areasi;
    xDss{iC} = xDsi;
    yDss{iC} = yDsi;
    magss{iC} = magsi;
    magAss{iC} = magAsi;
    bkss{iC} = bksi;
    Ls{iC} = L;

    %% not the end of this window, continue update the cache
    if pF < Ran0(2, iF)
        continue;
    end

    %% the end of this window, compute the cumulative score
    while iF <= nF && pF == Ran0(2, iF)
        head = RanC(1, iF);
        tail = RanC(2, iF);
        cen = RanC(3, iF);

% indC = circleIdx(head, tail, wS);
% L2s = cat(3, Ls{indC});
        L3s = L;

        %% re-compute the neightbor
        A3 = vdoSegL2A(L3s, mSeg);
        % A3 = vdoSegLS2A(L3s, S, mSeg);

        %% histogram distance
        idx = find(areas == 0);
        DF = computeHistDist(sparse(HstF), A3, idx);
        DM = computeHistDist(sparse(HstM), A3, idx);
        DO = computeHistDist(sparse(HstO), A3, idx);

        %% histogram saliency
        SHstF(:, iF) = computeSalHst(DF, A3, areas, siz);
        SHstM(:, iF) = computeSalHst(DM, A3, areas, siz);
        SHstO(:, iF) = computeSalHst(DO, A3, areas, siz);

        %% spatial prior saliency
        SPriS(:, iF) = computeSalPriSpa(areas, xDs, yDs, siz);
        SPriM(:, iF) = computeSalPriMag(areas, mags);
        SPriA(:, iF) = computeSalPriMag(areas, magAs);
        SPriB(:, iF) = computeSalPriMag(areas, bks);

        %% debug
        if iF == 54 && 0
            disp('here');
            % save('tmp.mat', 'F', 'P', 'HstO', 'A3', 'areas', 'idx', 'A', 'sHstMs', 'DM');
            save('tmp.mat', 'F', 'P', 'M', 'MA', 'bins', 'VX', 'VY');
        end

        iF = iF + 1;
    end
end
prCOut(nF0);

prOut;
