function [HstF, HstM, HstO, areas, xDs, yDs, L, mags, magAs, bks] = vdoSalVoxHst(hrVdo, hrFlow, hrAcc, hrBk, hrSeg, pF, mSeg, bins, XD, YD, magMa, magMaA)
% Computing video saliency based on spatio-temporal voxels.
%
% Input
%   hrVdo   -  video input handle
%   hrFlow  -  optical flow input handle
%   wsSmp   -  sampling
%   wsFlow  -  flow
%   wsAcc   -  acceleration
%   wsBk    -  background
%   wsSeg   -  segmentation
%   bins    -  bins for quantatize appearance and flow, {[8 16 16 4 8 9]}
%   wS      -  window for cumulating histogram for computing saliency
%   direS   -  window type,  'both' | 'left' | 'right'
%
% Output
%   SHstF   -  salience of apperance, mSeg x nF
%   SHstM   -  salience of magtitude, mSeg x nF
%   SHstO   -  salience of orientation, mSeg x nF
%   SPriS   -  salience of spatial prior, mSeg x nF
%   SPriM   -  salience of motion prior, mSeg x nF
%   SPriB   -  salience of background prior, mSeg x nF
%   SPriA   -  salience of acceleration, mSeg x nF
%
% History
%   create  -  Feng Zhou (zhfe99@gmail.com), 01-03-2009
%   modify  -  Feng Zhou (zhfe99@gmail.com), 07-27-2013

prIn('vdoSalVoxHst');

siz = stFld(hrSeg, 'siz');

% read frame
F0 = vdoR(hrVdo, pF);
F = imresize(F0, siz);

% read flow
mat = vdoR(hrFlow, pF);
[VX, VY] = stFld(mat, 'VX', 'VY');
VX = double(VX);
VY = double(VY);

% read acc
mat = vdoR(hrAcc, pF);
[DX, DY] = stFld(mat, 'DX', 'DY');
DX = double(DX);
DY = double(DY);

% mag
M = real(sqrt(VX .^ 2 + VY .^ 2));
M = M ./ (magMa + eps);
MA = real(sqrt(DX .^ 2 + DY .^ 2));
MA = MA ./ (magMaA + eps);

% background
mat = vdoR(hrBk, pF);
P = double(mat.P);

% segmentation
mat = vdoR(hrSeg, pF);
L = double(mat.L);
% S = mat.S;

% quantize
FQ = computeQuantMatrixImg(F, bins(1 : 4));
MQ = computeQuantMatrixFlow(M, bins(5));

% compute histogram
HstF = computeRegionHist_fast(FQ, L, prod(bins(1 : 4)), mSeg);
HstM = computeRegionHist_fast(MQ, L, bins(5), mSeg);
HstO = computeHof(VX, VY, L, bins(6), mSeg);
HstF = sparse(HstF);
HstM = sparse(HstM);
HstO = sparse(HstO);

% spatial prior of each region
[areas, xDs, yDs] = computeRegionSpa_fast(L, XD, YD, mSeg);
mags = computeRegionMag_fast(L, M, mSeg);
magAs = computeRegionMag_fast(L, MA, mSeg);
bks = computeRegionMag_fast(L, P, mSeg);

prOut;
