function [SHstF, SHstM, SSpa, HstF, HstM] = vdoSalVoxOld(hrVdo, hrFlow, wsFlow, wsSeg, bins, wF)
% Computing video saliency based on spatio-temporal voxels.
%
% Input
%   hrVdo   -  video input handle
%   hrFlow  -  optical flow input handle
%   wsFlow  -  flow
%   wsSeg   -  segmentation
%   bins    -  #bin, e.g., [8 16 16 4 8]
%   wF      -  window for smoothing flow
%
% Output
%   SHstF   -  salience of each semgent, mSeg x nF
%   SHstM   -  salience of each semgent, mSeg x nF
%   SSpa    -  salience of each semgent, mSeg x nF
%   HstF    -  histogram of image, mSeg x k
%   HstM    -  histogram of flow, mSeg x k
%
% History
%   create  -  Feng Zhou (zhfe99@gmail.com), 01-03-2009
%   modify  -  Feng Zhou (zhfe99@gmail.com), 07-18-2013

% video in
[nF, siz0] = stFld(hrVdo, 'nF', 'siz');

% flow normalization
mas = maxConv(wsFlow.mags, wF);

% segmentation in
hrSeg = vdoRIn(wsSeg.vdo, 'comp', 'mat');
[siz, nF] = stFld(hrSeg, 'siz', 'nF');
h = siz(1);
w = siz(2);

% segmentation
[cL, mSeg, A] = stFld(wsSeg, 'cL', 'mSeg', 'A');

% spatial prior
[XD, YD] = meshgrid(-w / 2 + 1 : w / 2, -h / 2 + 1 : h / 2);
XD = XD .^ 2;
YD = YD .^ 2;

% function option
prIn('vdoSalVox', 'nF %d, siz [%d %d], mSeg %d, bins [%d %d %d %d %d], wF %d', ...
     nF, siz(1), siz(2), mSeg, bins(1), bins(2), bins(3), bins(4), bins(5), wF);

% histogram
prCIn('frame', nF, .1);
for iF = 1 : nF
    prC(iF);

    %% frame
    F0 = vdoR(hrVdo, iF);

    %% flow
    V0 = vdoR(hrFlow, iF);
    VX0 = V0(:, :, 1);
    VY0 = V0(:, :, 2);
    M0 = real(sqrt(VX0 .^ 2 + VY0 .^ 2));
    M0 = M0 ./ (mas(iF) + eps);

    %% resize image
    F = imresize(F0, siz);
    M = imresize(M0, siz);
    M = max(M, 0);
    VX = imresize(VX0, siz);
    VY = imresize(VY0, siz);

    %% quantize
    FQ = computeQuantMatrix(F, bins(1 : 4));
    MQ = computeQuantMatrixFlow(M, bins(5));

    %% segmentation
    mat = vdoR(hrSeg, iF);
    L = double(mat.L);

    %% compute the color histogram
    HstFi = computeRegionHist_fast(FQ, L, prod(bins(1 : 4)), mSeg);
    HstMi = computeRegionHist_fast(MQ, L, bins(5), mSeg);

    HstMi = computeRegionHist_fast(VX, VY, L, bins(5), mSeg);

    %% spatial prior of each region
    [areasi, xDsi, yDsi] = computeRegionSpa_fast(L, XD, YD, mSeg);

    %% store
    if iF == 1
        HstF = HstFi;
        HstM = HstMi;
        areas = areasi;
        xDs = xDsi;
        yDs = yDsi;
    else
        HstF = HstF + HstFi;
        HstM = HstM + HstMi;
        areas = areas + areasi;
        xDs = xDs + xDsi;
        yDs = yDs + yDsi;
    end
end
prCOut(nF);

% compute histogram distance
DF = computeHistDist(HstF, A);
DM = computeHistDist(HstM, A);

% compute one superpixel-scale saliency map based on context analysis
[~, sHstFs] = computeOneScaleSmap(DF, A, areas, xDs, yDs, siz);
[~, sHstMs, sSpas] = computeOneScaleSmap(DM, A, areas, xDs, yDs, siz);

% apply to the entire video
SHstF = repmat(sHstFs, 1, nF);
SHstM = repmat(sHstMs, 1, nF);
SSpa = repmat(sSpas, 1, nF);

prOut;
