function [R, ss, mS] = maskRegSplitNext(wsReg, wsReg0, s0s, mS0)
% Split mask to disjoint regions.
%
% Input
%   s0s      -  label in last frame, 1 x mR0
%   mS0      -  #label up to the last frame
%   idxR2L0  -  region id -> label id, 1 x mR0
%   IdxL2R0  -  label id -> region id, mL0 x coRMa0
%   idxR2T0  -  region id -> label count, 1 x mR0
%   xCen0s   -  center x of each region, 1 x mR0
%   yCen0s   -  center y of each region, 1 x mR0
%
% Output
%   R        -  region matrix, h x w (uint16)
%   ss       -  label in current frame, 1 x mR
%   mS       -  #label up to the last frame
%   coRs     -  #region for each label, 1 x mR
%   idxR2L   -  region id -> label id, 1 x mR
%   IdxL2R   -  label id -> region id, mL x coRMa
%   idxR2T   -  region id -> label count, 1 x mR
%   xCens    -  center x of each region, 1 x mR
%   yCens    -  center y of each region, 1 x mR
%
% History
%   create  -  Feng Zhou (zhfe99@gmail.com), 11-08-2010
%   modify  -  Feng Zhou (zhfe99@gmail.com), 07-23-2013

% current frame
[R, mR, coRs, coRMa, idxR2L, IdxL2R, idxR2T, xCens, yCens] = stFld(wsReg, 'R', 'mR', 'coRs', 'coRMa', 'idxR2L', 'IdxL2R', 'idxR2T', 'xCens', 'yCens');

% previous frame
[R0, mR0, coR0s, coRMa0, idxR2L0, IdxL2R0, idxR2T0, xCen0s, yCen0s] = stFld(wsReg0, 'R', 'mR', 'coRs', 'coRMa', 'idxR2L', 'IdxL2R', 'idxR2T', 'xCens', 'yCens');

% distance
D = maskRegDst(IdxL2R, coRs, xCens, yCens, idxR2L0, xCen0s, yCen0s);

% loop each region in last frame
mR0 = length(xCen0s);
ss = zeross(1, mR);
cRs = zeros(1, length(xCen0s));
mS = mS0;
for cR0 = 1 : mR0
    %% region with the same label
    l = idxR2L0(cR0);

    coRc = coRs(l);
    cRs = IdxL2R(1 : coRc);

    %% no such label
    if coRc == 0
        continue;
    end

    %% closest region
    [~, iR] = min(D(cR0, 1 : coRc));
    cR = IdxL2R(l, iR);

    %% assign label
    s0 = s0s(cR0);
    ss(cR) = s0;
end

% non-visited region
for cR = 1 : mR
    if ss(cR) == 0
        mS = mS + 1;
        ss(cR) = mS;
    end
end
